
; = Info ===========================================================================================
; Title           : hover effect easy PB
; --------------------------------------------------------------------------------------------------
;
; Demo: Professor Bernd, 2020-08-28.
;
; Der Code basiert auf der hervorragenden Lsung von "Health" (AutoIt DE Forum).
; The code is based on the excellent solution of "Health" (AutoIt DE Forum).
;
; Autor der ursprnglichen Funktion / Author of the original function: Health, 2008-04-30.
; Quelle: https://autoit.de/thread/6087-control-hover-effekt-ohne-fehler/
;
; Modifikation / Modification: Professor Bernd, 2020-08-18 ...
; ==================================================================================================

#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>  ; $SS_CENTER, $SS_BLACKFRAME, $SS_WHITEFRAME, ...
#include <WindowsConstants.au3> ; $WS_BORDER, ...

Opt("MustDeclareVars", 1)

Example()

Func Example()

  Global $hGui = GUICreate("hover effect easy PB", 400, 386)

  ; Controls ohne Hover-Effekt. / Controls without hover effect.
  ; ----------------------------------------------------------------------------
  ;
  Global $LabelTitle_1 = GUICtrlCreateLabel("Controls without hover effect", 34, 16, 328, 33, BitOR($SS_CENTER, $SS_CENTERIMAGE))
  GUICtrlSetFont($LabelTitle_1, 10, 700)
  ;
  Global $Input1 = GUICtrlCreateInput("0", 32, 64, 129, 21)
  Global $Updown1 = GUICtrlCreateUpdown($Input1)
  Global $Checkbox1 = GUICtrlCreateCheckbox("Checkbox1", 32, 96, 97, 17)
  Global $Checkbox2 = GUICtrlCreateCheckbox("Checkbox2", 32, 128, 97, 17)
  ;
  Global $Group1 = GUICtrlCreateGroup("Group1", 184, 56, 185, 97)
  Global $Radio1 = GUICtrlCreateRadio("Radio1", 200, 80, 113, 17)
  Global $Radio2 = GUICtrlCreateRadio("Radio2", 200, 104, 113, 17)
  Global $Radio3 = GUICtrlCreateRadio("Radio3", 200, 128, 113, 17)
  ;
  GUICtrlCreateGroup("", -99, -99, 1, 1)
  Global $Label1 = GUICtrlCreateLabel("OK", 116, 176, 80, 28, BitOR($SS_CENTER, $SS_CENTERIMAGE))
  GUICtrlSetBkColor(-1, 0xE6E6E6) ; 0xE6E6E6: helles grau / light gray
  Global $Label2 = GUICtrlCreateLabel("Apply", 208, 176, 80, 28, BitOR($SS_CENTER, $SS_CENTERIMAGE))
  GUICtrlSetBkColor(-1, 0xE6E6E6)
  Global $Label3 = GUICtrlCreateLabel("Cancel", 300, 176, 80, 28, BitOR($SS_CENTER, $SS_CENTERIMAGE))
  GUICtrlSetBkColor(-1, 0xE6E6E6)

  ; Controls mit Hover-Effekt. / Controls with hover effect.
  ; ----------------------------------------------------------------------------
  Global $LabelTitle_2 = GUICtrlCreateLabel("Controls with hover effect", 34, 240, 328, 33, BitOR($SS_CENTER, $SS_CENTERIMAGE))
  GUICtrlSetFont($LabelTitle_2, 10, 700)
  ;
  ; Labels mit Rahmen als Hover-Effekt.
  ; Labels with frame as hover effect.
  Global $g_idLblMain_OK = GUICtrlCreateLabel("OK", 116, 326, 80, 28, BitOR($SS_CENTER, $SS_CENTERIMAGE))
  GUICtrlSetBkColor(-1, 0xD7EDFF) ; 0xD7EDFF: helles blau / light blue
  Global $g_idLblMain_Apply = GUICtrlCreateLabel("Apply", 208, 326, 80, 28, BitOR($SS_CENTER, $SS_CENTERIMAGE))
  GUICtrlSetBkColor(-1, 0xD7EDFF)
  Global $g_idLblMain_Cancel = GUICtrlCreateLabel("Cancel", 300, 326, 80, 28, BitOR($SS_CENTER, $SS_CENTERIMAGE))
  GUICtrlSetBkColor(-1, 0xD7EDFF)

  ; Rahmen-Label / frame label
  ; ----------------------------------------------------------------------------
  ; Label, das den Highlight-Rahmen fr die Label-Buttons darstellt.
  ; Wichtig:   Highlight-Rahmen muss NACH den Label-Buttons erstellt werden (damit er sichtbar wird)!
  ; --------   (Bei Labels funktioniert der Rahmen sehr gut, bei anderen Controls evtl. nicht so gut.)
  ;
  ; Label, which represents the highlight frame for the label buttons.
  ; Important: Highlight frame must be created AFTER the label buttons (so that it becomes visible)!
  ; ---------- (With labels the frame works very well, with other controls maybe not so well.)
  Global $g_idLbl_FrameHL_Black = GUICtrlCreateLabel("", -1000, -1000, 40, 40, $SS_BLACKFRAME) ; -1000: hides the Ctrl
  Global $g_idLbl_FrameHL_White = GUICtrlCreateLabel("", -1000, -1000, 40, 40, $SS_WHITEFRAME) ; -1000: hides the Ctrl
  Global $g_idLbl_FrameHL ; Helper variable for black & white frame labels.

  GUISetState(@SW_SHOW, $hGui)

  ; Loop until the user exits.
  While 1

    Switch GUIGetMsg()
      Case $GUI_EVENT_CLOSE, $g_idLblMain_Cancel, $Label3
        ExitLoop
    EndSwitch

    ; Zeige den Hover-Effekt fr bestimmte Controls.
    ; Show the hover effect for certain controls.
    HoverEffect_FrameHL(0) ; Highlight frame for Labels.  Param: 0 = Black, 1 = White.
  WEnd
EndFunc ; ==> Example


; ******************************************************************************
;   Hover-Effekt Highlight-Rahmen / Hover effekt highlight frame
; ******************************************************************************

; ==============================================================================
; HoverEffect_FrameHL
;
; Routine zum Zeigen eines Hover-Effekts fr bestimmte Controls.
; Routine to show a hover effect for certain controls.
;
; Hover effect:
;
;   Frame wird um das Label angezeigt, ber dem sich der Mouse-Cursor befindet.
;   Frame is displayed around the label over which the mouse cursor is hovering.
;
; Tipp: Bei Labels funktioniert der Rahmen sehr gut, bei anderen Controls evtl. nicht so gut.
; Hint: With labels the frame works very well, with other controls maybe not so well.
; ------------------------------------------------------------------------------
Func HoverEffect_FrameHL($_iBlackOrWhite = 0) ; Param: 0 = Black, 1 = White

  If $_iBlackOrWhite = 0 Then
    $g_idLbl_FrameHL = $g_idLbl_FrameHL_Black
  Else
    $g_idLbl_FrameHL = $g_idLbl_FrameHL_White
  EndIf
  Local Static $idHoveredCtrl_Old
  Local $aCInfo = GUIGetCursorInfo($hGui) ; it's important that the gui variable is setted in this function!

  If $idHoveredCtrl_Old <> $aCInfo[4] Then

    ; Highlight-Frame verstecken. / Hide the highlight frame.
    If ID_IsInList_HoverFrame($idHoveredCtrl_Old) Then
      ControlHide($hGui, "", $g_idLbl_FrameHL)
    EndIf

    ; Highlight-Frame um das Label anzeigen. / Show the highlight frame on the label.
    ; $aCInfo[4] = ID of the control that the mouse cursor is hovering over (or 0 if none).
    If ID_IsInList_HoverFrame($aCInfo[4]) Then
      Local $aPos = ControlGetPos($hGui, "", $aCInfo[4])
      GUICtrlSetPos($g_idLbl_FrameHL, $aPos[0], $aPos[1], $aPos[2], $aPos[3])
      ControlShow($hGui, "", $g_idLbl_FrameHL)
    EndIf

    $idHoveredCtrl_Old = $aCInfo[4]
  EndIf
EndFunc ; ==> HoverEffect_FrameHL


; ==============================================================================
; ID_IsInList_HoverFrame
;
; Routine zum Ermitteln, ob die bergebene Control-ID in der Liste ist.
; Routine to determine whether the passed Control ID is in the list.
; ------------------------------------------------------------------------------
Func ID_IsInList_HoverFrame($_iID)
  Switch $_iID
    ; Liste der Controls fr den Hover-Effekt. / List of controls for the hover effect.
    Case $g_idLblMain_OK, $g_idLblMain_Apply, $g_idLblMain_Cancel, $g_idLbl_FrameHL, $LabelTitle_2
      Return True
    Case Else
      Return False
  EndSwitch
EndFunc ; ==> ID_IsInList_HoverFrame
